﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Domain.Share
{
    /// <summary>
    /// 模块枚举
    /// </summary>
    public enum ModuleTypeEnum
    {
        /// <summary>
        /// 计划内访视
        /// </summary>
        InPlanSubjectVisit = 0,

        /// <summary>
        /// 计划外访视
        /// </summary>
        OutPlanSubjectVisit = 1,

        ///// <summary>
        ///// 阅片期
        ///// </summary>
        //Read = 2,

        /// <summary>
        /// 全局阅片
        /// </summary>
        Global = 3,

        /// <summary>
        /// 裁判
        /// </summary>
        Referee = 4,

        /// <summary>
        /// 肿瘤学
        /// </summary>
        Oncology = 5,


    }

    public enum ReadingSetType
    {
        /// <summary>
        /// 影像阅片
        /// </summary>
        ImageReading = 0,

        /// <summary>
        /// 肿瘤阅片
        /// </summary>
        TumorReading = 1,
    }

    /// <summary>
    /// 阅片期范围
    /// </summary>
    public enum ReadingScopeEnum
    {
        /// <summary>
        /// 全部
        /// </summary>
        All = 0,

        /// <summary>
        /// 根据Site
        /// </summary>
        Site = 1,
    }





    /// <summary>
    /// 状态
    /// </summary>
    public enum ReadingPeriodStatus
    {
        /// <summary>
        /// 未生效
        /// </summary>
        NotTakeEffect = 0,

        /// <summary>
        /// 已生效
        /// </summary>
        TakeEffect = 1,

        /// <summary>
        /// 已撤销
        /// </summary>
        Revocation = 2,
    }

    /// <summary>
    /// 上传角色
    /// </summary>
    public enum UploadRole
    {
        /// <summary>
        /// CRC
        /// </summary>
        CRC = 0,

        /// <summary>
        /// PM
        /// </summary>
        PM = 1,
    }

    /// <summary>
	/// 上传方式
	/// </summary>
	public enum ClinicalUploadType
    {
        /// <summary>
        /// 表格
        /// </summary>
        [Display(Name = "表格")]
        Table = 0,

        /// <summary>
        /// PDF
        /// </summary>
        [Display(Name = "PDF")]
        PDF = 1,

    }

    /// <summary>
    /// 数据类型
    /// </summary>
    public enum ClinicalDataType
    {
        /// <summary>
        /// 既往局部治疗史
        /// </summary>
        MedicalHistory = 0,

        /// <summary>
        /// 影像模态（PET临床信息）
        /// </summary>
        PET = 1,

        /// <summary>
        /// 胸水
        /// </summary>
        Hydrothorax = 2,
    }

    public enum LesionType
    {
        /// <summary>
        /// 靶病灶
        /// </summary>
        TargetLesion = 0,

        /// <summary>
        /// 非靶病灶
        /// </summary>
        NonTargetLesions = 1,

        /// <summary>
        /// 新病灶
        /// </summary>
        NewLesions = 1,

    }


    public enum OrganType
    {
        /// <summary>
        /// 非靶和新病灶
        /// </summary>
        OffTargetAndNewLesions = 0,


        /// <summary>
        /// 结节靶病灶
        /// </summary>
        NodularTargetLesions = 1,


        /// <summary>
        /// 非结节靶病灶
        /// </summary>
        NonNodularTargetLesions = 2,
    }

    /// <summary>
    /// 临床分组级别
    /// </summary>
    public enum ClinicalLevel
    {
        /// <summary>
        /// 受试者
        /// </summary>
        [DisplayAttribute(Name = "受试者")]
        Subject = 0,

        /// <summary>
        /// 访视
        /// </summary>
        [DisplayAttribute(Name = "访视")]
        SubjectVisit = 1,

        /// <summary>
        /// 影像学阅片
        /// </summary>
        [DisplayAttribute(Name = "影像学阅片")]
        ImageRead = 2,

        /// <summary>
        /// 肿瘤学阅片
        /// </summary>
        [DisplayAttribute(Name = "肿瘤学阅片")]
        OncologyRead = 3,
    }

    public enum QuestionMark
    {
        /// <summary>
        /// 长径
        /// </summary>
        MajorAxis=0,

        /// <summary>
        /// 短径
        /// </summary>
        ShortAxis = 1,

        /// <summary>
        /// 是否淋巴结
        /// </summary>
        IsLymph = 2,

        /// <summary>
        /// 自增Id
        /// </summary>
        AutoId=3,
    }

    public enum QuestionType
    {
        /// <summary>
        /// 靶病灶径线之和（SOD）
        /// </summary>
        SOD = 0,

        /// <summary>
        /// 非淋巴结靶病灶长径之和
        /// </summary>
        SumOfDiameter = 1,

        /// <summary>
        /// 与基线SOD相比变化量（mm）
        /// </summary>
        SODChange = 2,

        /// <summary>
        /// 与基线访视相比SOD变化百分比
        /// </summary>
        SODPercent = 3,

        /// <summary>
        /// 与整个访视期间最低点相比增加的值（mm）
        /// </summary>
        LowestIncrease = 4,

        /// <summary>
        /// 与整个访视期间最低点相比增加的百分比
        /// </summary>
        LowPercent = 5,

        /// <summary>
        /// 整个访视期间最低点访视名称
        /// </summary>
        LowVisit = 6,

        /// <summary>
        /// 是否存在非淋巴结靶病灶
        /// </summary>
        IsLymphTarget = 7,

        /// <summary>
        /// 是否存在淋巴结靶病灶且该病灶比上一访视短径增加5MM以上
        /// </summary>
        IsAddFive = 8,

        /// <summary>
        /// 被评估为NE的单个靶病灶
        /// </summary>
        NETarget = 9,

        /// <summary>
        /// 靶病灶评估
        /// </summary>
        TargetLesion = 10,

        /// <summary>
        /// 非靶病灶评估
        /// </summary>
        NoTargetLesion = 11,

        /// <summary>
        /// 是否存在新病灶
        /// </summary>
        NewLesions = 12,

        /// <summary>
        /// 整体肿瘤评估
        /// </summary>
        Tumor = 13,
    }

    /// <summary>
    /// 裁判阅片问题类型
    /// </summary>
    public enum JudgeReadingQuestionType
    {
        /// <summary>
        /// 问题
        /// </summary>
        Question=1,

        /// <summary>
        /// 全局修改
        /// </summary>
        GlobalChange=2,

        /// <summary>
        /// 访视点注释
        /// </summary>
        VisitRemark = 3,
    }

    public enum JudgeTypeEnum
    {
        None=0,

        /// <summary>
        /// 答案不相同
        /// </summary>
        AnswerDisaffinity=1,

        /// <summary>
        /// 答案分组
        /// </summary>
        AnswerGroup=2,

        /// <summary>
        /// 答案组合
        /// </summary>
        AnswerCombination=3,


    }

    /// <summary>
    /// 是否必填
    /// </summary>
    public enum IsRequired
    {
        /// <summary>
        /// 必填
        /// </summary>
        Required = 0,

        /// <summary>
        /// 依赖父问题
        /// </summary>
        Rely = 1,

        /// <summary>
        /// 不必填
        /// </summary>
        NotRequired = 2,

     
    }


    /// <summary>
    /// 是否关联
    /// </summary>
    public enum IsDepend
    {
        /// <summary>
        /// 是
        /// </summary>
        Yes = 0,

        /// <summary>
        /// 依赖父问题
        /// </summary>
        Rely = 1,

        /// <summary>
        /// 否
        /// </summary>
        No = 2,

    }

    /// <summary>
    /// 是否显示问题
    /// </summary>
    public enum ShowQuestion
    {
        /// <summary>
        /// 显示
        /// </summary>
        Show=0,

        /// <summary>
        /// 依赖父问题
        /// </summary>
        Rely=1,

        /// <summary>
        /// 不显示
        /// </summary>
        Hide = 2,

    }

    /// <summary>
    /// 阅片状态
    /// </summary>
    public enum ReadingStatusEnum
    {
        /// <summary>
        /// 影像上传（未提交）
        /// </summary>
        ImageNotSubmit = 0,

        /// <summary>
        /// 影像质控(未质控通过 --已提交)
        /// </summary>
        ImageQuality = 1,

        /// <summary>
        /// 一致性核查（未一致性核查通过 ---已质控通过）
        /// </summary>
        ConsistencyCheck = 2,

        /// <summary>
        /// 任务分配 (一致性核查通过 --未分配)
        /// </summary>
        TaskAllocate = 3,

        /// <summary>
        /// 影像阅片中（任务已分配--未阅片完成）
        /// </summary>
        ImageReading = 4,

        /// <summary>
        /// 阅片完成
        /// </summary>
        ReadCompleted = 5,
    }


    public enum ReadingClinicalDataStatus
    {
        WaitUpload=0,

        HaveUploaded=1,

        HaveChecked=2,

        HaveSigned=3

    }

    public enum ClinicalFileType
    {
        //既往手术
        PreviousSurgery=1,

        //既往放疗历史
        PreviousHistory=2,

        //其他
        PreviousOther=3

    }
}
